"""
Configuration module for the TwitterAPI.io MCP server.

This module contains constants, configuration values, and logging setup.
It loads environment variables from .env files using the utils module.
"""

import os
import logging
from typing import Dict, List, Any, Optional

from twitterapi.utils import load_environment

# Load environment variables from .env files
load_environment()

# Configure logging
logging.basicConfig(
    level=logging.INFO,
    format="%(asctime)s - %(name)s - %(levelname)s - %(message)s",
    handlers=[logging.StreamHandler()]
)
logger = logging.getLogger("twitterapi-mcp")

# Environment variable configuration
def get_env_var(name: str, default: Optional[str] = None, required: bool = False) -> Optional[str]:
    """
    Get an environment variable with validation.
    
    Args:
        name: Name of the environment variable
        default: Default value if environment variable is not set
        required: Whether the environment variable is required
        
    Returns:
        The environment variable value or default
        
    Raises:
        ValueError: If the environment variable is required but not set
    """
    value = os.environ.get(name, default)
    if required and value is None:
        logger.error(f"{name} environment variable is required")
        raise ValueError(f"{name} environment variable is required")
    return value

# API Configuration
API_KEY = get_env_var("TWITTER_API_KEY", required=True)
BASE_URL = get_env_var("TWITTER_API_BASE_URL", "https://api.twitterapi.io")
LOG_LEVEL = get_env_var("LOG_LEVEL", "INFO")
CACHE_TTL = int(get_env_var("CACHE_TTL", "3600"))  # Default: 1 hour
MAX_TWEETS = int(get_env_var("MAX_TWEETS", "100"))

# Set log level from environment
log_level = getattr(logging, LOG_LEVEL.upper(), logging.INFO)
logger.setLevel(log_level)

# AI coding influencers to monitor by default
DEFAULT_AI_INFLUENCERS: List[str] = [
    "AndrewYNg",       # Andrew Ng
    "karpathy",        # Andrej Karpathy
    "sama",            # Sam Altman
    "AnthropicAI",     # Anthropic
    "OpenAI",          # OpenAI
    "GeminiApp",       # Google Gemini
    "GosuCoder",       # GosuCoder
    "_philschmid",     # Philipp Schmid
    "cjzafir",         # Chris Zafir
    "ryolu_",          # Ryan Lu
    "ericzakariasson", # Eric Zakariasson
    "jasonzhou1993",   # Jason Zhou
    "MatthewBerman",   # Matthew Berman
    "roo_code",        # Roo Code
    "cline",           # Cline
    "aisdk",           # AI SDK
    "windsurf_ai",     # Windsurf AI
    "dr_cintas",       # Dr. Cintas
    "PrajwalTomar_",   # Prajwal Tomar
    "Saboo_Shubham_",  # Shubham Saboo
    "cursor_ai",       # Cursor AI
    "rowancheung",     # Rowan Cheung
    "shivsakhuja"      # Shiv Sakhuja
]

# Topic extraction keywords
AI_TOPICS: List[str] = [
    "ai", "ml", "python", "coding", "llm", "development", 
    "tensorflow", "pytorch", "langchain", "whisper", "gpt",
    "transformer", "vector", "embedding", "inference", "fine-tuning"
]